package com.engine.hansang.action;

import com.time.util.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.Util;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.Property;
import weaver.soa.workflow.request.RequestInfo;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 更新婚假余额
 * <p>
 * 处理方式：
 * 1、拟稿表单增加“登记时间”（必填），“附件”（必填）、是否初婚（必填）；
 * 2、开发：满足 员工自助更新流程办结时，更新员工婚假余额。
 * 条件：流程上自助信息类型选择“婚姻状况”
 * 在职期间登记结婚的（登记日期与入职日期判断），初婚15天，再婚3天，自动更新员工假期余额中
 *
 * @author wangj
 * @version 1.00版本
 * @Date 2023/9/4
 */

public class UpdateMarriageLeaveBalance implements Action {
    @Override
    public String execute(RequestInfo requestInfo) {

        Map mainnInfo = new HashMap(1000);
        Property[] property = requestInfo.getMainTableInfo().getProperty();
        for (int i = 0; i < property.length; i++) {
            mainnInfo.put(property[i].getName().toLowerCase(), Util.null2String(property[i].getValue()));
        }
        //自助信息类型
        String zzxxlx = Util.null2String(mainnInfo.get("zzxxlx"));
        //婚姻状况
        String hyzk = Util.null2String(mainnInfo.get("hyzk"));

        try {


            //已婚
            if ("1".equals(zzxxlx) && "1".equals(hyzk)) {
                int days = 0;
                //是否初婚
                String sfch = Util.null2String(mainnInfo.get("sfch"));
                //登记日期
                String djrq = Util.null2String(mainnInfo.get("djrq"));
                //申请人
                String sqr = Util.null2String(mainnInfo.get("sqr"));
                //入职日期
                String rzrq = getRzrq(sqr);

                //判断结婚登记日期在入职期间的
                boolean flag = compareDateLessEquation(rzrq, djrq);
                if (flag) {
                    if ("1".equals(sfch)) {
                        //再婚3天
                        days = 3;
                    } else {
                        //0 或 空默认初婚 15天
                        days = 15;
                    }
                    if (days > 0) {
                        updateLeaveBalance(sqr, days);
                    }
                }

            }

        } catch (Exception e) {
            requestInfo.getRequestManager().setMessagecontent("接口异常:"+e.getMessage());
            return Action.FAILURE_AND_CONTINUE;
        }
        return Action.SUCCESS;
    }

    /**
     * @Description: 比较日期小于等于
     * @Param:
     * @return:
     * @Author: wangj
     */
    private boolean compareDateLessEquation(String start, String end) {
        boolean flag = false;
        if ("".equals(start) || "".equals(end)) {
            return false;
        }
        Date sdate = DateUtil.parseDate(start, "yyyy-MM-dd");
        Date edate = DateUtil.parseDate(end, "yyyy-MM-dd");
        if (sdate.getTime() < edate.getTime()) {
            flag = true;
        }
        return flag;
    }

    private String getRzrq(String id) {
        String date = "";
        RecordSet rs = new RecordSet();
        String sql = "select workcode,lastname,companystartdate from hrmresource where id = ?";
        rs.executeQuery(sql, id);
        while (rs.next()) {
            date = Util.null2String(rs.getString("companystartdate"));
        }
        return date;
    }

    private void updateLeaveBalance(String sqr, int days) {
        String sql = "select count(1) as sl from kq_balanceofleave where leaveRulesId = 32 and resourceid = ?";
        RecordSet rs = new RecordSet();
        int count = 0;
        rs.executeQuery(sql,sqr);
        while (rs.next()){
            count = Util.getIntValue(rs.getString("sl"));
        }

        String year = DateUtil.getCurrentTime("yyyy");

        if(count > 0){
            sql = "update kq_balanceofleave set leaveRulesId = ?, resourceId = ?, belongYear = ?, baseAmount = ?, extraAmount = ?, usedAmount = ?, baseAmount2 = ?, extraAmount2 = ?, usedAmount2 = ?, status = ?,tiaoxiuamount = ?,isDelete = ? where resourceid = ? and leaveRulesId = ?";
            rs.executeUpdate(sql,"32",sqr,year,days,"0","0","0","0","0","0","0","0",sqr,"32");
        }else{
            sql = "insert into kq_balanceofleave (leaveRulesId, resourceId, belongYear, baseAmount, extraAmount, usedAmount, baseAmount2, extraAmount2, usedAmount2, status,tiaoxiuamount) values (?,?,?,?,?,?,?,?,?,?,?)";
            rs.executeUpdate(sql,"32",sqr,year,days,"0","0","0","0","0","0","0");
        }

    }
}
