package com.engine.hansang.job;

import com.time.util.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * 更新人员lastname字段信息
 *
 * 开发需求：
 * 1、定时任务执行，lastname值改为：工号-姓名
 * 2、离职日期有值时，lastname值改为：工号-姓名（R）。离职日期字段field82，工作信息中
 *
 * @author wangj
 * @version 1.00版本
 * @Date 2023/9/4
 */

public class UpdateHrmLastNameJob extends BaseCronJob {
    @Override
    public void execute() {
        String date = DateUtil.getCurrentTime("yyyy-MM-dd");
        RecordSet rs = new RecordSet();
        String sql = "select a.id,workcode,lastname,field82 as lzrq from hrmresource a left join cus_fielddata b on a.id = b.id where b.scope = 'HrmCustomFieldByInfoType' and scopeid = 3";

        rs.execute(sql);
        while (rs.next()){
            String newLastname = "";
            String id = Util.null2String(rs.getString("id"));
            String workcode = Util.null2String(rs.getString("workcode"));
            String lastname = Util.null2String(rs.getString("lastname"));
            String lzrq = Util.null2String(rs.getString("lzrq"));

            if(lastname.contains("-")){
                String[] lastnameArr = lastname.split("-");
                lastname = lastnameArr[lastnameArr.length-1];
                lastname = lastname.replaceAll("\\(R\\)","");
            }
            newLastname = workcode+"-"+lastname;
            if(!"".equals(lzrq)){
                //当前日期>=离职日期的吧
                boolean flag = compareDate(lzrq,date);
                if(flag){
                    newLastname = newLastname+"(R)";
                }
            }
            updateLastName(id,newLastname);
        }

    }

    private void updateLastName(String id,String lastname){
        RecordSet rs = new RecordSet();
        String sql = "update hrmresource set lastname = ? where id = ?";
        rs.executeUpdate(sql,lastname,id);
    }

    //比较时间大小
    public static boolean compareDate(String start, String end) {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
        try {
            Date date1 = simpleDateFormat.parse(start);
            Date date2 = simpleDateFormat.parse(end);
            if (date1.getTime() <= date2.getTime()) {
                return true;
            }
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        return false;
    }

}
