package com.engine.leshanvc.util;

import com.engine.kq.biz.KQBalanceOfLeaveBiz;
import com.engine.kq.biz.KQGroupMemberComInfo;
import com.engine.kq.biz.KQHolidaySetComInfo;
import weaver.common.DateUtil;
import weaver.general.Util;

/**
 * @author:dxfeng
 * @createTime: 2023/04/19
 * @version: 1.0
 */
public class HsDateUtil {


    /**
     * 根据开始日期，假期余额，自动计算结束日期
     *
     * @param resourceId     人员ID
     * @param addDays        假期余额
     * @param startDate      假期开始日期
     * @param includeHoliday 假期是否包含节假期
     * @return
     */
    public static String addDay(String resourceId, Integer addDays, String startDate, boolean includeHoliday) {
        int i = 0;
        String tempDate = startDate;
        boolean isCurrentDate = false;
        while (i < addDays) {
            if (isCurrentDate) {
                tempDate = DateUtil.addDate(tempDate, 1);
            } else {
                isCurrentDate = true;
            }
            boolean holiday = isHoliday(resourceId, tempDate, false);
            if (includeHoliday || !holiday) {
                i++;
            }
        }

        return tempDate;
    }

    /**
     * 获取假期余额
     *
     * @param resourceId   人员ID
     * @param newLeaveType 假期类型
     * @param startDate    假期开始时间
     * @return
     */
    public static Integer getHolidayAmount(String resourceId, String newLeaveType, String startDate) {
        String restAmount = KQBalanceOfLeaveBiz.getRestAmount(resourceId, newLeaveType, startDate);
        String substring = "0".equals(restAmount) ? restAmount : restAmount.substring(0, restAmount.lastIndexOf('.'));
        return Integer.parseInt(substring);
    }

    /**
     * @param resourceId     人员ID
     * @param date           判断日期
     * @param containWeekend 是否包含周末
     * @return
     */
    public static boolean isHoliday(String resourceId, String date, boolean containWeekend) {
        boolean flag = false;

        /*获取考勤组的ID，因为考勤组有有效期，所以需要传入日期*/
        KQGroupMemberComInfo kqGroupMemberComInfo = new KQGroupMemberComInfo();
        String groupId = kqGroupMemberComInfo.getKQGroupId(resourceId, date);

        int changeType = getChangeType(groupId, date);
        /*如果需要包含周末，判断传入的日期是不是周末*/
        if (containWeekend && changeType != 2 && (DateUtil.getWeek(date) == 6 || DateUtil.getWeek(date) == 7)) {
            flag = true;
        }
        if (changeType == 1) {
            flag = true;
        }
        return flag;
    }

    /**
     * @param groupId 考勤组的ID
     * @param date    判断日期
     * @return
     */
    public static int getChangeType(String groupId, String date) {
        //默认没有设置过节假日(节假日设置的类型：1-公众假日、2-调配工作日、3-调配休息日)
        KQHolidaySetComInfo kqHolidaySetComInfo = new KQHolidaySetComInfo();
        return Util.getIntValue(kqHolidaySetComInfo.getChangeType(groupId, date), -1);
    }
}
