package com.engine.workflow.biz.requestForm;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Properties;

import com.engine.workflow.util.FileUtils;
import org.apache.axis.client.Call;
import org.apache.axis.client.Service;

import weaver.general.BaseBean;
import weaver.general.GCONST;
import weaver.general.Util;

/**
 * Description: html表单转pdf
 *
 * @author Mcr
 *
 */
public class Html2Pdf extends BaseBean{

    /**
     * 将离线html表单转成pdf格式
     *
     * @param htmlPath 离线表单路径
     * @param pdfPath 生成的pdf文件路径
     * @param filename 文件名
     * @param zipPath zip文件路径
     * @return
     */
    public boolean getPdf(String htmlPath,String pdfPath,String filename,String zipPath){
        boolean result = true;

        boolean isWindows = isWindows();
        boolean useLocalServer = isUseLocalServer(isWindows);

        if(useLocalServer){//使用本地插件转pdf useLocalServer
            String cmd = getToolPath(isWindows) + " --enable-local-file-access " + htmlPath + " " + pdfPath;

            long l1 = System.currentTimeMillis();
            writeLog("开始导出pdf");
            writeLog("执行脚本：" + cmd);
            Process p = null;
            BufferedInputStream err = null;
            BufferedReader errBr = null;
            try {
                Runtime run = Runtime.getRuntime();
                p = run.exec(cmd);//启动另一个进程来执行命令
                err = new BufferedInputStream(p.getErrorStream());
                errBr = new BufferedReader(new InputStreamReader(err));
                String lineStr = "";
                while ((lineStr = errBr.readLine()) != null){
                    writeLog(lineStr);
                }
                //检查命令是否执行失败。
                try {
                    if (p.waitFor()!=0) {
                        if(p.exitValue()==1)//p.exitValue()==0表示正常结束，1：非正常结束
                            writeLog("执行过程中出现异常");
                    }
                }catch (InterruptedException e){
                    e.printStackTrace();
                }
                writeLog("执行成功");

            } catch (Exception e) {
                result = false;
                e.printStackTrace();
            }finally {
                try {
                    errBr.close();
                    err.close();
                    if (p != null) {
                        p.destroy();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            long l2 = System.currentTimeMillis();
            writeLog("开始导出结束，耗时："+ (l2-l1));
        }else{//使用window中转服务转pdf
            String windowsServer = getPropValue("form2pdf_e9", "windowsServer");
            if(!"".equals(windowsServer)){
                File zipFile = new File(zipPath);
                try {
                    String fileString = FileUtils.file2String(zipFile);
                    String pdfString = callService(fileString, windowsServer, filename);
                    if(!"".equals(pdfString)){
                        byte[] string2Byte = FileUtils.string2Byte(pdfString);
                        result = FileUtils.saveFile(pdfPath, string2Byte);
                    }else{
                        result = false;
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }else{
                result = false;
                writeLog("form2pdf_e9.properties中的windowsServer参数未配置，请检查！");
            }
        }

        return result;
    }

    /**
     * 是否使用本地插件
     *
     * @param isWindows
     * @return
     */
    private boolean isUseLocalServer(boolean isWindows){
        boolean result = true;
        if(!isWindows){//linux系统
            if(getConvertType() == 3)
                result = false;//使用windows中转服务
        }
        return result;
    }

    /**
     * 获取转换类型
     * @return
     */
    private int getConvertType(){
        int convertType = Util.getIntValue(getPropValue("form2pdf_e9", "convertType"),1);
        return convertType;
    }

    /**
     * 判断当前服务器是否为windows系统
     *
     * @return
     */
    public boolean isWindows(){
        boolean isWindows = true;
        Properties properties = System.getProperties();
        String os = properties.getProperty("os.name").toLowerCase();

        if(os.indexOf("windows") > -1){
            isWindows = true;
        }else{
            isWindows = false;
        }

        return isWindows;
    }

    /**
     * 获取wkhtmltopdf地址
     * @return
     */
    private String getToolPath(boolean isWindows){
        String toolPath = GCONST.getRootPath() + "wkhtmltopdf";
        if(isWindows){//Windows系统
            toolPath = "cmd /c " + toolPath + File.separator + "windows" + File.separator + "bin" + File.separator + "wkhtmltopdf.exe";
        }else{//Linux系统
            int convertType = getConvertType();
            if(convertType == 1){//使用ecology自带插件
                toolPath +=  File.separator + "linux" + File.separator + "bin" + File.separator + "./wkhtmltopdf";
            }else if(convertType == 2){//使用自行安装插件
                toolPath = Util.null2String(getPropValue("form2pdf_e9", "wkhtml2xPath")) + File.separator + "bin" + File.separator + "./wkhtmltopdf";
            }
        }

        return toolPath;
    }

    public static void wKConvertLinux(String whktmlExe,String htmlUrl, String pdfFilePath) {
        try {
            //ProcessBuilder pb = new ProcessBuilder(whktmlExe,"--print-media-type", "--margin-left", "10mm", "--margin-right", "10mm", htmlUrl, pdfFilePath);
            ProcessBuilder pb = new ProcessBuilder(whktmlExe, htmlUrl, pdfFilePath);
            pb.redirectErrorStream(true);
            Process process = pb.start();
            BufferedReader errstreamReader = new BufferedReader(new InputStreamReader(process.getInputStream()));
            String line = errstreamReader.readLine();
            while (line != null) {
                line = errstreamReader.readLine();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

    }

    /**
     * 调用windows中转服务
     * @param fileString
     * @param windowsServer
     * @param filename
     * @return
     */
    private String callService(String fileString,String windowsServer,String filename){
        String result = "";
        try {
            String endpoint = windowsServer + "/services/PdfService";
            Service service = new Service();
            Call call = (Call) service.createCall();
            call.setTargetEndpointAddress(endpoint);
            call.setOperationName("convertPdf");//方法
            call.addParameter("fileString", org.apache.axis.encoding.XMLType.XSD_STRING, javax.xml.rpc.ParameterMode.IN);//接口的参数
            call.addParameter("filename", org.apache.axis.encoding.XMLType.XSD_STRING, javax.xml.rpc.ParameterMode.IN);
            call.setReturnType(org.apache.axis.encoding.XMLType.XSD_STRING);//设置返回类型


            result = (String)call.invoke(new Object[]{fileString,filename});
        }catch (Exception e) {
            writeLog("调用window中转服务【html->pdf】失败！");
        }

        return result;
    }

}

