package weaver.interfaces.hansang.job;


import com.engine.kq.biz.KQHolidaySetBiz;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.general.TimeUtil;
import weaver.interfaces.schedule.BaseCronJob;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.UUID;

/**
 * @author wanxq
 * @date 2023年09月18日10:17:00
 * @desc
 */
public class AttendanceDaysCalculateJob extends BaseCronJob {

    private BaseBean log = new BaseBean();

    @Override
    public void execute() {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDate yearStartDate = LocalDate.parse(DateUtil.getYear()+"-01-01", formatter);
        for(int i=0;i<12;i++){
            LocalDate startDate = yearStartDate.plusMonths(i);
            LocalDate endDate = startDate.withDayOfMonth(startDate.lengthOfMonth());
            log.writeLog("AttendanceDaysCalculateJob startDate==>"+startDate+"  endDate==>"+endDate);

            int workdays = 0;
            long daysDiff = ChronoUnit.DAYS.between(startDate, endDate)+1;
            for(int j=0;j<daysDiff;j++){
                String tempdate = startDate.plusDays(j).toString();

                //判断是否是法定节假日、周末
                boolean isHoliday = false;
                int changeType = KQHolidaySetBiz.getChangeType("2",tempdate);
                log.writeLog("AttendanceDaysCalculateJob tempdate==>"+tempdate+" changeType==>"+changeType+" weekend==>"+DateUtil.getWeek(tempdate));
                if (changeType != 2 && changeType != 1 && (DateUtil.getWeek(tempdate) == 6 || DateUtil.getWeek(tempdate) == 7)) {
                    isHoliday = true;
                }
                if (changeType == 3) {
                    isHoliday = true;
                }

                if(!isHoliday){
                    workdays++;
                    log.writeLog("AttendanceDaysCalculateJob workday==>"+tempdate);
                }
            }

            insertData(workdays,startDate.toString());
        }
    }

    public void insertData(int workdays,String month){
        RecordSet rs = new RecordSet();
        //删除建模表中当前算薪月的数据
        rs.executeUpdate("delete from uf_ykcqr where xzszy=? ",month);

        int formmodeid = getModeIdByName("应扣出勤日");
        String modeuuid = UUID.randomUUID().toString();
        int modedatacreater = 1;
        String modedatacreatedate = TimeUtil.getCurrentDateString();
        String modedatacreatetime = TimeUtil.getOnlyCurrentTimeString();
        String insertSql = " insert into uf_ykcqr(xzszy,yzkts,formmodeid,modedatacreater,modedatacreatedate," +
                "modedatacreatetime,modeuuid) values(?,?,?,?,?,?,?) ";
        rs.executeUpdate(insertSql,month,workdays,
                formmodeid,modedatacreater,modedatacreatedate,modedatacreatetime,modeuuid);
    }




    /**
     * 根据建模名称获取建模id,建模权限同步的使用
     * @param modeName 建模名称
     * @return int 建模id
     */
    public static int getModeIdByName(String modeName){
        RecordSet rs = new RecordSet();
        String sql = " SELECT ID FROM MODEINFO WHERE MODENAME = ? ";
        rs.executeQuery(sql, modeName);
        if (rs.next()) {
            return rs.getInt("id");
        }
        return -1;
    }



}
