package weaver.interfaces.hansang.job;


import com.cloudstore.dev.api.bean.MessageBean;
import com.cloudstore.dev.api.bean.MessageType;
import com.cloudstore.dev.api.util.Util_Message;
import com.engine.kq.biz.KQWorkTime;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

/**
 * @author wanxq
 * @date 2023年08月17日10:45:22
 * @desc
 */
public class KqAbnormalRemind extends BaseCronJob {

    @Override
    public void execute() {
        RecordSet rs = new RecordSet();
        String currentDate = DateUtil.getCurrentDate();
        String currentMonth = currentDate.substring(0,7);
        String sql = "  select a.id,field112,b.field138 from hrmresource a,cus_fielddata b " +
                " where a.id=b.id and a.status<=3 and b.scopeid=3 ";
        rs.execute(sql);
        while(rs.next()){
            String userid = Util.null2String(rs.getString("id"));
            String field112 = Util.null2String(rs.getString("field112"));
            String field138 = Util.null2String(rs.getString("field138"));

            if(!"0".equals(field112) && !"0".equals(field138)){
                continue;
            }

            boolean isExist = hasAbnormalData(userid,currentMonth,currentDate);
            if(isExist){
                Set<String> userIdList = new HashSet<>();
                userIdList.add(userid);

                //内容
                String context = getMsg(userid,currentMonth,currentDate);
                sendRemindMsg(userIdList,context);
            }
        }
    }


    /**
     * 发送提醒
     * @param userIdList
     */
    public void sendRemindMsg(Set<String> userIdList,String context){
        try {
            MessageType messageType = MessageType.newInstance(2022061096);
            String title = "考勤异常提醒";
            MessageBean messageBean = Util_Message.createMessage(messageType, userIdList, title, context, "", "");
            messageBean.setCreater(1);
            Util_Message.store(messageBean);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public String getMsg(String userid,String kqmonth,String currentDate){
        KQWorkTime kqWorkTime = new KQWorkTime();
        String lastMonthEndDay = TimeUtil.getLastMonthEndDay();
        String msg = "温馨提示:<br />您的考勤有异常<br/>";
        RecordSet rs = new RecordSet();
        String sql = "  select a.kqdate,a.absenteeism,a.absenteeismmins,a.forgotcheck,a.forgotBeginWorkCheck,a.belate,a.leaveeearly,b.signintime,b.signouttime " +
                " from kq_format_total a  " +
                " left join kq_format_detail b on a.resourceid=b.resourceid and a.kqdate=b.kqdate " +
                " where (a.kqdate like '"+kqmonth+"%' or a.kqdate=? ) and a.resourceid=? " +
                " and (absenteeism>0 or forgotcheck>0 or forgotBeginWorkCheck >0 or a.belate>0 or a.leaveeearly>0) and a.kqdate!=? ";
        rs.executeQuery(sql,lastMonthEndDay,userid,currentDate);
        while (rs.next()){
            String kqdate = Util.null2String(rs.getString("kqdate"));
            String signintime = Util.null2String(rs.getString("signintime"));
            String signouttime = Util.null2String(rs.getString("signouttime"));

            signintime = "".equals(signintime)?"无":signintime;
            signouttime = "".equals(signouttime)?"无":signouttime;

            //旷工
            int absenteeism = Util.getIntValue(rs.getString("absenteeism"),0);
            //漏签
            int forgotcheck = Util.getIntValue(rs.getString("forgotcheck"),0);
            int forgotBeginWorkCheck = Util.getIntValue(rs.getString("forgotBeginWorkCheck"),0);
            //迟到
            int belate = Util.getIntValue(rs.getString("belate"),0);
            //早退
            int leaveeearly = Util.getIntValue(rs.getString("leaveeearly"),0);

            String kqResult = "";
            if( absenteeism>0){
                kqResult = "旷工";
            }

            if( forgotcheck>0 || forgotBeginWorkCheck>0){
                kqResult = "漏签";
            }

            if( belate>0){
                kqResult = "迟到";
            }

            if( leaveeearly>0){
                kqResult = "早退";
            }
            msg += "日期："+kqdate+"，上班打卡时间："+signintime+",下班打卡时间："+signouttime+"，考勤结果："+kqResult+"<br/>";
        }
        return msg;
    }

    /**
     * 判断是否有异常数据
     * @param userid
     * @param kqmonth
     * @return
     */
    public boolean hasAbnormalData(String userid,String kqmonth,String currentDate){
        RecordSet rs = new RecordSet();
        String lastMonthEndDay = TimeUtil.getLastMonthEndDay();
        String sql = "  select * from kq_format_total  " +
                " where (kqdate like '"+kqmonth+"%' or kqdate=?)  and resourceid=? " +
                " and (absenteeism>0 or forgotcheck>0 or forgotBeginWorkCheck >0 or belate>0 or leaveeearly>0) and kqdate!=? ";
        rs.executeQuery(sql,lastMonthEndDay,userid,currentDate);
        return rs.next();
    }


    public static void main(String[] args) {
        System.out.println(TimeUtil.getLastMonthEndDay());
    }


}
