package weaver.interfaces.hansang.job;


import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.text.DecimalFormat;
import java.time.format.DateTimeFormatter;

/**
 * @author wanxq
 * @date 2023年08月17日14:17:51
 * @desc
 */
public class ParentalLeaveCalculateJob extends BaseCronJob {

    @Override
    public void execute() {

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        DecimalFormat df = new DecimalFormat("#.00");
        //当前日期减出生日期小于3年才计算育儿假
        RecordSet rs = new RecordSet();
        String sql = "  select ry,csrq,xm,DATE_ADD(csrq, INTERVAL 3 YEAR) jsrq,companystartdate " +
                " from uf_jt a " +
                " left join hrmresource b on a.ry=b.id " +
                " where gx=2 and DATE_ADD(csrq, INTERVAL 3 YEAR) >=curdate()  order by csrq and status<=3 ";
        rs.executeQuery(sql);
        while (rs.next()){
            String ry = Util.null2String(rs.getString("ry"));
            String csrq = Util.null2String(rs.getString("csrq"));
            String jsrq = Util.null2String(rs.getString("jsrq"));
            String companystartdate = Util.null2String(rs.getString("companystartdate"));



            String birthYear = csrq.substring(0,4);
            String endYear = jsrq.substring(0,4);
            String currentYear = DateUtil.getYear();

            String currentYearFirstDay = DateUtil.getFirstDayOfYear();
            String currentYearLastDay = DateUtil.getLastDayOfYear();

            double jqts = 0;

            // companystartdate: 2023-03-13  csrq:2023-06-01

            //入职日期小于子女出生日期，育儿假基数：当前年12月31日减子女出生日期/365*10为育儿假基数(天)
            if(DateUtil.compDate(csrq,companystartdate)<0){
                if(birthYear.equals(currentYear)){
                    int diffDays = DateUtil.dayDiff(currentYearLastDay,csrq);
                    jqts = Double.valueOf(df.format((double)diffDays/365*10));
                }else if(endYear.equals(currentYear)){
                    int diffDays = DateUtil.dayDiff(jsrq,currentYearFirstDay);
                    jqts = Double.valueOf(df.format((double)diffDays/365*10));
                }else{
                    jqts = 10;
                }

            }else{//入职日期大于等于子女出生日期，育儿假基数：当前年12月31日减入职日期/365*10为育儿假基数(天)
                if(birthYear.equals(currentYear)){
                    int diffDays = DateUtil.dayDiff(currentYearLastDay,companystartdate);
                    jqts = Double.valueOf(df.format((double)diffDays/365*10));
                }else if(endYear.equals(currentYear)){
                    int diffDays = DateUtil.dayDiff(jsrq,currentYearFirstDay);
                    jqts = Double.valueOf(df.format((double)diffDays/365*10));
                }else{
                    jqts = 10;
                }

            }
            rs.writeLog("ry==>"+ry+" csrq=>"+csrq+" jsrq==>"+jsrq+" companystartdate=>"+companystartdate+" jqts==>"+jqts);
            setLeaveTypeDays(ry,jqts,currentYear);
        }
    }

    /**
     * 设置人员的育儿假天数
     * @param ry
     * @param jqts
     * @param belongyear
     */
    public void setLeaveTypeDays(String ry, double jqts,String belongyear) {
        RecordSet rs = new RecordSet();
        String sql = " update kq_balanceofleave set baseAmount=? " +
                " where leaveRulesId=? and resourceId=? and belongyear=? ";
        rs.writeLog("setLeaveTypeDays sql==>"+sql+" jqts==>"+jqts+" ry==>"+ry+" belongyear==>");
        rs.executeUpdate(sql,jqts,36,ry,belongyear);
    }


    public static void main(String[] args) {
        DecimalFormat df = new DecimalFormat("#.00");
        double jqts = 0;

        String ry = "601";
        String csrq = "2021-06-29";
        String jsrq = "2024-06-29";
        String companystartdate = "2011-03-21";

        String birthYear = csrq.substring(0,4);
        String endYear = jsrq.substring(0,4);
        String currentYear = DateUtil.getYear();

        String currentYearFirstDay = DateUtil.getFirstDayOfYear();
        String currentYearLastDay = DateUtil.getLastDayOfYear();


        // companystartdate: 2023-03-13  csrq:2023-06-01

        //入职日期小于子女出生日期，育儿假基数：当前年12月31日减子女出生日期/365*10为育儿假基数(天)
        if(DateUtil.compDate(csrq,companystartdate)<0){
            if(birthYear.equals(currentYear)){
                int diffDays = DateUtil.dayDiff(currentYearLastDay,csrq);
                jqts = Double.valueOf(df.format((double)diffDays/365*10));
            }else if(endYear.equals(currentYear)){
                int diffDays = DateUtil.dayDiff(jsrq,currentYearFirstDay);
                jqts = Double.valueOf(df.format((double)diffDays/365*10));
            }else{
                jqts = 10;
            }

        }else{//入职日期大于等于子女出生日期，育儿假基数：当前年12月31日减入职日期/365*10为育儿假基数(天)
            if(birthYear.equals(currentYear)){
                int diffDays = DateUtil.dayDiff(currentYearLastDay,companystartdate);
                jqts = Double.valueOf(df.format((double)diffDays/365*10));
            }else if(endYear.equals(currentYear)){
                int diffDays = DateUtil.dayDiff(jsrq,currentYearFirstDay);
                jqts = Double.valueOf(df.format((double)diffDays/365*10));
            }else{
                jqts = 10;
            }

        }
        System.out.println(jqts);
    }



}
