package weaver.interfaces.hansang.job;


import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.formmode.setup.ModeRightInfo;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.util.UUID;

/**
 * @author wanxq
 * @date 2023年08月08日15:30:32
 * @desc 内推奖金计算
 */
public class RecommendBonusCalculateJob extends BaseCronJob {


    @Override
    public void execute() {
        RecordSet rs = new RecordSet();
        String sql = "select distinct c.field73 from hrmresource b left join cus_fielddata c on b.id= c.id \n" +
                "where  status<=3 and c.field73 is not null and c.field35 in(2,3)  and c.SCOPEID = -1";
        rs.execute(sql);
        while(rs.next()){
            //推荐人
            int userid = Util.getIntValue(rs.getString("field73"));
            calBonus(userid);
        }
    }

    /**
     * 计算推荐人奖金
     * @param userid
     */
    public void calBonus(int userid){
        try {
            double tjje = 0;
            RecordSet rs = new RecordSet();
            String lastMonthEndDay = TimeUtil.getLastMonthEndDay();
            //若推荐人为（陈涛、笪莹、徐奇3位HRBP），则只发放一次满60天的奖金
            boolean ishrbp = isHrbp(userid);
            String sql = "  select a.id,companystartdate,certificatenum from cus_fielddata a,hrmresource b " +
                    " where a.id=b.id and status<=3 and datediff('"+lastMonthEndDay+"',companystartdate)>=60 and field73=? ";
            rs.executeQuery(sql,userid);
            while(rs.next()){
                int id = Util.getIntValue(rs.getString("id"));
                String certificatenum = Util.null2String(rs.getString("certificatenum"));
                String companyStartDate = Util.null2String(rs.getString("companystartdate"));

                int days = DateUtil.dayDiff(companyStartDate,lastMonthEndDay)+1;
                int cs = (days/30)-1;
                String entryDate = DateUtil.addDate(lastMonthEndDay,-180);
                int entryTimes = getEntryTimes(certificatenum,entryDate);
                if(ishrbp){
                    if(cs==1 && entryTimes<=1){
                        tjje += 200;
                    }
                }else{
                    if(cs>0 && cs<=36 && entryTimes<=1){
                        tjje += 200;
                    }
                }
            }

            if(tjje>0){
                //更新内推机制表
                String month = lastMonthEndDay.substring(0,7);
                insertData(userid,tjje,month);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    public void insertData(int userid,double tjje,String month){
        RecordSet rs = new RecordSet();
        //删除建模表中当前算薪月的数据
        rs.executeUpdate("delete from uf_ntjz where tjr=? and xzszy=? ",userid,month);

        int formmodeid = getModeIdByName("内推机制");
        String modeuuid = UUID.randomUUID().toString();
        int modedatacreater = 1;
        String modedatacreatedate = TimeUtil.getCurrentDateString();
        String modedatacreatetime = TimeUtil.getOnlyCurrentTimeString();
        String insertSql = " insert into uf_ntjz(xzszy,tjr,tjje,xzszyqsrq,formmodeid,modedatacreater,modedatacreatedate," +
                "modedatacreatetime,modeuuid) values(?,?,?,?,?,?,?,?,?) ";
        rs.executeUpdate(insertSql,month,userid,tjje,month+"-01",
                formmodeid,modedatacreater,modedatacreatedate,modedatacreatetime,modeuuid);

        rs.executeQuery("select id from uf_ntjz where modeuuid=?",modeuuid);
        rs.next();
        int id = rs.getInt("id");
        if(id>0){
            ModeRightInfo right = new ModeRightInfo();
            right.editModeDataShare(userid, formmodeid, id);
        }

    }


    /**
     * 根据建模名称获取建模id,建模权限同步的使用
     * @param modeName 建模名称
     * @return int 建模id
     */
    public static int getModeIdByName(String modeName){
        RecordSet rs = new RecordSet();
        String sql = " SELECT ID FROM MODEINFO WHERE MODENAME = ? ";
        rs.executeQuery(sql, modeName);
        if (rs.next()) {
            return rs.getInt("id");
        }
        return -1;
    }


    /**
     * 查询180天入职次数
     * @param idCard
     * @param companyStartDate
     * @return
     */
    public int getEntryTimes(String idCard,String companyStartDate){
        int entryTimes = 0;
        RecordSet rs = new RecordSet();
        String sql = " select count(1) cs from hrmresource where certificatenum=? and companystartdate>=? ";
        rs.executeQuery(sql,idCard,companyStartDate);
        if(rs.next()){
            entryTimes = Util.getIntValue(rs.getString("cs"),0);
        }
        return entryTimes;
    }

    /**
     * 判断是否是HRBP
     * @param resourceid
     * @return
     */
    public boolean isHrbp(int resourceid){
        RecordSet rs = new RecordSet();
        rs.executeQuery(" select 1 from hrmroles a,hrmrolemembers b where a.id=b.roleid and rolesmark='推荐机制_HRBP' and resourceid=? ",resourceid);
        return rs.next();
    }




}
