package weaver.interfaces.hansang.job;


import com.engine.kq.biz.KQWorkTime;
import org.apache.commons.lang.StringUtils;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.formmode.setup.ModeRightInfo;
import weaver.general.TimeUtil;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @author wanxq
 * @date 2023年08月07日14:22:14
 * @desc 病假薪资比例计算
 */
public class SickLeaveSalaryCalculateJob extends BaseCronJob {
    private String salaryMonth;

    public String getSalaryMonth() {
        return salaryMonth;
    }

    public void setSalaryMonth(String salaryMonth) {
        this.salaryMonth = salaryMonth;
    }

    @Override
    public void execute() {
        RecordSet rs = new RecordSet();
        String fromDate = DateUtil.getFirstDayOfMonthToString();
        String toDate = DateUtil.getLastDayOfMonthToString();

        if(StringUtils.isNotBlank(salaryMonth)){
            fromDate = DateUtil.getFirstDayOfMonth(this.salaryMonth+"-01");
            toDate = DateUtil.getLastDayOfMonth(this.salaryMonth+"-01");
        }

        //查询算薪月有请病假的人员
        String sql = "  select distinct resourceid from kq_flow_split_leave \n" +
                " where newleavetype in(4,27) and belongdate >=? and belongdate<=? ";
        rs.executeQuery(sql,fromDate,toDate);
        while(rs.next()){
            int userid = Util.getIntValue(rs.getString("resourceid"));
            calContinueLeaveDays(userid,fromDate,toDate);
        }
    }


    /**
     * 查询人员的病假信息
     * @param resourceid
     * @param calEndDate
     * @return
     */
    public Map<String, String> getLeaveInfo(int resourceid,String calEndDate){
        Map<String, String> leaveInfo = new HashMap<>();
        String calStartDate = TimeUtil.getDateString(DateUtil.addMonth(TimeUtil.getCalendar(calEndDate),-7));
        RecordSet rs = new RecordSet();
        String sql = "  select belongdate,requestid from kq_flow_split_leave " +
                " where resourceid=? and belongdate >=? and belongdate<=? and newleavetype in(4,27) ";
        rs.executeQuery(sql,resourceid,calStartDate,calEndDate);
        while(rs.next()){
            String belongdate = Util.null2String(rs.getString("belongdate"));
            String requestid = Util.null2String(rs.getString("requestid"));
            leaveInfo.put(belongdate,requestid);
        }
       return leaveInfo;
    }

    public void calContinueLeaveDays(int resourceid,String salaryFirstDate,String salaryLastDate){
        //查询算薪月最后一次请假日期
        RecordSet rs = new RecordSet();
        String sql = "  select max(belongdate) lastdate from kq_flow_split_leave  " +
                " where resourceid=? and belongdate >=? and belongdate<=? ";
        rs.executeQuery(sql,resourceid,salaryFirstDate,salaryLastDate);
        if(rs.next()){
            String lastDate = Util.null2String(rs.getString("lastdate"));
            //查询从请假日开始往期6个月的请假信息
            Map<String, String> leaveInfo = getLeaveInfo(resourceid,lastDate);

            double level = 0;
            for(int i=1;i<7;i++){
                String calStartDate = TimeUtil.getDateString(DateUtil.addMonth(TimeUtil.getCalendar(lastDate),-1));
                double leaveRate = getLeaveRate(resourceid,calStartDate,lastDate,leaveInfo);
                rs.writeLog("SickLeaveSalaryCalculateJob resourceid==》"+resourceid+" calStartDate==>"+calStartDate+" calEndDate==>"+lastDate+" leaveRate==>"+leaveRate);
                level += leaveRate;
                if(leaveRate!=1){
                    break;
                }
                lastDate = calStartDate;
            }

            //如果满6个月要判断是否是6个月以上（往前找到第一个工作日判断是否是请假）
            if(level==6){
                KQWorkTime kqWorkTime = new KQWorkTime();
                boolean flag = true;
                while(flag){
                    lastDate = DateUtil.addDate(lastDate,-1);
                    boolean isWorkDay = kqWorkTime.isWorkDay(resourceid + "", lastDate);
                    if(isWorkDay){
                        flag = false;
                    }
                    rs.writeLog(" lastdate==>"+lastDate+" isWorkDay==>"+isWorkDay);
                }
                if(leaveInfo.containsKey(lastDate)){
                    level ++;
                }
            }

            double kkbl = 0;
            if(level<=1){
                kkbl = 0.3;
            }else if(level<=3){
                kkbl = 0.4;
            }else if(level<=6){
                kkbl = 0.5;
            }else{
                kkbl = 1;
            }

            //更新病假扣款比例表
            String month = salaryLastDate.substring(0,7);
            insertData(resourceid,kkbl,month);
        }

    }

    /**
     * 判断日期区间内是否连续休病假
     * @param resourceid
     * @param fromDate
     * @param toDate
     * @param leaveInfo
     * @return
     */
    public double getLeaveRate(int resourceid,String fromDate,String toDate,Map<String, String> leaveInfo){
        double leaveRate = 0;
        KQWorkTime kqWorkTime = new KQWorkTime();
        int workdays = 0;
        int leavedays = 0;
        boolean interrupt = false;
        int diffdays = DateUtil.dayDiff(fromDate,toDate);
        for(int i=0;i<diffdays;i++){
            String tempdate = DateUtil.addDate(toDate,-i);
            boolean isWorkDay = kqWorkTime.isWorkDay(resourceid + "", tempdate);
            if(isWorkDay){
                workdays ++;
            }

            //判断是否中断，中断后不再计算连续请假天数
            if(isWorkDay){
                if(leaveInfo.containsKey(tempdate) && !interrupt){
                    leavedays++;
                }else {
                    interrupt = true;
                }
            }
        }
        leaveRate = workdays == 0?0:(double)leavedays/workdays;
        return leaveRate;
    }


    public void insertData(int userid,double kkbl,String month){
        RecordSet rs = new RecordSet();
        //删除建模表中当前算薪月的数据
        rs.executeUpdate("delete from uf_bjkkbl where ry=? and xzszy=? ",userid,month);

        int formmodeid = getModeIdByName("病假扣款比例");
        String modeuuid = UUID.randomUUID().toString();
        int modedatacreater = 1;
        String modedatacreatedate = TimeUtil.getCurrentDateString();
        String modedatacreatetime = TimeUtil.getOnlyCurrentTimeString();
        String insertSql = " insert into uf_bjkkbl(ry,kkbl,xzszy,xzszyqsrq,formmodeid,modedatacreater,modedatacreatedate," +
                "modedatacreatetime,modeuuid) values(?,?,?,?,?,?,?,?,?) ";
        rs.executeUpdate(insertSql,userid,kkbl,month,month+"-01",
                formmodeid,modedatacreater,modedatacreatedate,modedatacreatetime,modeuuid);

        rs.executeQuery("select id from uf_bjkkbl where modeuuid=?",modeuuid);
        rs.next();
        int id = rs.getInt("id");
        if(id>0){
            ModeRightInfo right = new ModeRightInfo();
            right.editModeDataShare(userid, formmodeid, id);
        }

    }


    /**
     * 根据建模名称获取建模id,建模权限同步的使用
     * @param modeName 建模名称
     * @return int 建模id
     */
    public static int getModeIdByName(String modeName){
        RecordSet rs = new RecordSet();
        String sql = " SELECT ID FROM MODEINFO WHERE MODENAME = ? ";
        rs.executeQuery(sql, modeName);
        if (rs.next()) {
            return rs.getInt("id");
        }
        return -1;
    }


}
