package weaver.interfaces.leshanvc.cronjob;

import org.apache.commons.collections.CollectionUtils;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.Util;
import weaver.interfaces.schedule.BaseCronJob;

import java.util.ArrayList;
import java.util.List;

/**
 * 同步离职信息计划任务
 *
 * @author:dxfeng
 * @createTime: 2023/03/03
 * @version: 1.0
 */
public class SyncResignJob extends BaseCronJob {
    /**
     * 建模表审批状态：已审批
     */
    private static final String PASS_STATUS = "2";

    /**
     * hrmresource表，离职状态
     */
    private static final String RESIGN_STATUS = "5";


    @Override
    public void execute() {
        RecordSet rs = new RecordSet();
        List<ResignInfo> resignInfoList = new ArrayList<>();

        rs.executeQuery("select lzry,jhlzrq,qdlzrq,lzyyxz  from uf_lzgl a inner join hrmresource b on a.lzry =b.ID and b.STATUS != ? where spzt = ? and qdlzrq < ? ", RESIGN_STATUS, PASS_STATUS, DateUtil.getCurrentDate());
        while (rs.next()) {
            resignInfoList.add(new ResignInfo(rs.getString("lzry"), rs.getString("jhlzrq"), rs.getString("qdlzrq"), rs.getString("lzyyxz")));
        }
        if (CollectionUtils.isNotEmpty(resignInfoList)) {
            for (ResignInfo resignInfo : resignInfoList) {
                // 更新人员卡片自定义字段
                rs.executeUpdate("update cus_fielddata set field31 = ?, field82 = ?, field50 = ? where scope ='HrmCustomFieldByInfoType' and scopeid ='3' and id = ? ", resignInfo.getPlanDate(), resignInfo.getResignDate(), resignInfo.getResignReason(), resignInfo.getResourceId());
                // 更新人员为离职状态
                rs.executeUpdate("update hrmresource set status = ? where id = ? ", RESIGN_STATUS, resignInfo.getResourceId());
            }
        }
    }


    private static class ResignInfo {

        /**
         * 离职人员
         */
        private String resourceId;
        /**
         * 计划离职日期
         */
        private String planDate;
        /**
         * 实际离职日期
         */
        private String resignDate;
        /**
         * 离职原因
         */
        private String resignReason;

        public ResignInfo(String resourceId, String planDate, String resignDate, String resignReason) {
            this.resourceId = resourceId;
            this.planDate = planDate;
            this.resignDate = resignDate;
            this.resignReason = resignReason;
        }

        public String getPlanDate() {
            return Util.null2String(planDate);
        }

        public void setPlanDate(String planDate) {
            this.planDate = planDate;
        }

        public String getResignDate() {
            return Util.null2String(resignDate);
        }

        public void setResignDate(String resignDate) {
            this.resignDate = resignDate;
        }

        public String getResignReason() {
            return Util.null2String(resignReason);
        }

        public void setResignReason(String resignReason) {
            this.resignReason = resignReason;
        }

        public String getResourceId() {
            return resourceId;
        }

        public void setResourceId(String resourceId) {
            this.resourceId = resourceId;
        }
    }


}
