package weaver.interfaces.leshanvc.workflow.action;

import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.interfaces.workflow.action.Action;
import weaver.sm.Util;
import weaver.soa.workflow.request.Cell;
import weaver.soa.workflow.request.RequestInfo;
import weaver.soa.workflow.request.DetailTable;
import weaver.soa.workflow.request.Row;
import weaver.workflow.request.RequestManager;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author ml
 * @Date 2023/3/20 17:44
 * @Description 批量加班申请流程
 * @Since version-1.0
 */
public class BatchWorkOverTimeAction implements Action {

    @Override
    public String execute(RequestInfo requestInfo) {
        List<HrmInfo> list = new ArrayList<>();
        RequestManager requestManager = requestInfo.getRequestManager();
        int requestId = requestManager.getRequestid();
        int billId = requestManager.getBillid();//表单数据ID
        //取明细数据
        try {
            DetailTable[] detailTable = requestInfo.getDetailTableInfo().getDetailTable();// 获取所有明细表
            if (detailTable.length > 0) {
                // 指定明细表
                for (DetailTable dt : detailTable) {
                    Row[] s = dt.getRow();// 当前明细表的所有数据,按行存储
                    // 指定行
                    for (Row r : s) {
                        Cell[] c = r.getCell();// 每行数据再按列存储
                        HrmInfo hrmInfo = new HrmInfo();
                        // 指定列
                        for (Cell c1 : c) {
                            String name = c1.getName();// 明细字段名称
                            String value = c1.getValue();// 明细字段的值
                            switch (name) {
                                case "jbry":
                                    hrmInfo.setResId(value);
                                    break;
                                case "jbsch":
                                    hrmInfo.setDuration(Double.parseDouble(value));
                                    break;
                                case "dyyljjbxs":
                                    hrmInfo.setMonthOverTime(Double.parseDouble(value));
                                    break;
                                case "dyljjbcexs":
                                    hrmInfo.setOverTimeAllowance(Double.parseDouble(value));
                                default:
                                    break;
                            }
                        }
                        list.add(hrmInfo);
                    }
                }
            }
            //平均加班时长
            double avgDuration = list.stream().mapToDouble(HrmInfo::getDuration).average().orElse(0d);
            Boolean gtFlag = false;// 当月累计加班 大于2h
            Boolean betweenFlag = false;// 2h<=平均<当月累计超额
            Boolean ltFlag = avgDuration < 2 ? true : false;// 小于2h
            for (HrmInfo h : list) {
                gtFlag = h.getDuration() + h.getMonthOverTime() >= h.getOverTimeAllowance() ? true : false;
                if (gtFlag) {
                    break;
                }
                betweenFlag = avgDuration >= 2 && avgDuration < h.getOverTimeAllowance() ? true : false;
                if (betweenFlag) {
                    break;
                }
            }
            // 加班时长审批（流程字段）
            String jbscsp = "";
            if (gtFlag) {
                jbscsp = "2";
            } else if (betweenFlag && StringUtils.isBlank(jbscsp)) {
                jbscsp = "1";
            } else if (ltFlag) {
                jbscsp = "0";
            }
            RecordSet rs = new RecordSet();
            rs.executeUpdate("update formtable_main_76 set jbscsp = ? where requestid = ? ", jbscsp, requestId);
        } catch (Exception e) {
            requestInfo.getRequestManager().setMessageid("11111" + requestInfo.getRequestid() + "22222");
            requestInfo.getRequestManager().setMessagecontent("【批量加班申请流程action】报错，请联系管理员！");
            return Action.FAILURE_AND_CONTINUE;
        }
        return Action.SUCCESS;
    }

        private static class HrmInfo {
            private String resId;
            //时长
            private double duration;
            //加班额度
            private double overTimeAllowance;
            //当月加班时长
            private double monthOverTime;

            public String getResId() {
                return resId;
            }

            public void setResId(String resId) {
                this.resId = resId;
            }

            public double getDuration() {
                return duration;
            }

            public void setDuration(double duration) {
                this.duration = duration;
            }

            public double getOverTimeAllowance() {
                return overTimeAllowance;
            }

            public void setOverTimeAllowance(double overTimeAllowance) {
                this.overTimeAllowance = overTimeAllowance;
            }

            public double getMonthOverTime() {
                return monthOverTime;
            }

            public void setMonthOverTime(double monthOverTime) {
                this.monthOverTime = monthOverTime;
            }
        }

}
