package weaver.interfaces.leshanvc.workflow.action;

import com.engine.kq.biz.KQOvertimeRulesBiz;
import com.engine.kq.util.KQDurationCalculatorUtil;
import com.engine.leshanvc.enums.MessageContentEnum;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.general.Util;
import weaver.interfaces.leshanvc.workflow.util.FunctionInterfaceMessage;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.RequestInfo;

import com.cloudstore.dev.api.bean.MessageBean;
import com.cloudstore.dev.api.bean.MessageType;
import com.cloudstore.dev.api.util.Util_Message;

import java.io.IOException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Author weaver_cl
 * @Description:
 * @Date 2023/2/10
 * @Version V1.0
 **/
public class MessagePushWeChatWorkAction implements Action {

    private static String messageType;

    private static String title;

    private static String linkUrl;

    private static String linkMobileUrl;



    @Override
    public String execute(RequestInfo requestInfo) {
        RecordSet rs = new RecordSet();
        FunctionInterfaceMessage functionInterfaceMessage = new FunctionInterfaceMessage();
        String tableName = requestInfo.getRequestManager().getBillTableName();
        String requestId = requestInfo.getRequestid();

        rs.executeQuery("select resourceId from " + tableName + " where requestid = ?", requestId);
        rs.next();
        String resourceId = Util.null2String(rs.getString("resourceId"));
        Set<String> userIdList = new HashSet<>();
        userIdList.add(resourceId);

        Map<String, String> map = getFirstAndLastDayMonth();
        double hour = getLastFlowOverTime(map.get("firstDay"), map.get("lastDay"), resourceId);
        List<MessageContentEnum> list = Arrays.asList(MessageContentEnum.ONE, MessageContentEnum.TWO, MessageContentEnum.THREE,
                MessageContentEnum.FOUR, MessageContentEnum.FIVE);

        Optional<MessageContentEnum> optional = list.stream().filter(messageContentEnum -> rangeInDefined(hour, messageContentEnum.getMin(), messageContentEnum.getMax())).findAny();
        MessageContentEnum messageContentEnum = optional.orElse(null);
        if (Objects.nonNull(messageContentEnum)) {
            String context = functionInterfaceMessage.doOperation(messageContentEnum.getValue(), 1);
            messagePush(context, userIdList);
        }

        if (isContinuousWork(resourceId)) {
            String context = functionInterfaceMessage.doOperation(null, 2);
            messagePush(context, userIdList);
        }


        return SUCCESS;
    }


    /**
     * 消息推送
     *
     * @param context
     * @param userIdList
     */
    private static void messagePush(String context, Set<String> userIdList) {

        MessageType message = MessageType.newInstance(Integer.parseInt(messageType));
        try {
            MessageBean messageBean = Util_Message.createMessage(message, userIdList, title, context, linkUrl, linkMobileUrl);
            messageBean.setCreater(1);
            //message.setBizState("0");// 需要修改消息为已处理等状态时传入,表示消息最初状态为待处理
            Util_Message.store(messageBean);

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 加班时长（个人本月）
     *
     * @param fromDate
     * @param toDate
     * @param resourceid
     */
    private static double getLastFlowOverTime(String fromDate, String toDate, String resourceid) {
        RecordSet rs = new RecordSet();
        DecimalFormat df = new DecimalFormat("0.00");
        //当前加班单位
        int uintType = KQOvertimeRulesBiz.getMinimumUnit();
        //当前天跟小时计算关系
        double hoursToDay = KQOvertimeRulesBiz.getHoursToDay();
        rs.executeQuery("select sum(cast(duration_min as decimal(18,4))) as val from kq_flow_overtime where" +
                " belongdate >= ? and belongdate <= ? and resourceid = ?", fromDate, toDate, resourceid);
        rs.next();
        double value = rs.getDouble("val") < 0 ? 0 : rs.getDouble("val");
        if (uintType == 3 || uintType == 5 || uintType == 6) {//按小时计算
            value = Util.getDoubleValue(KQDurationCalculatorUtil.getDurationRound(value / (60.0) + ""));
        } else {//按天计算
            value = Util.getDoubleValue(KQDurationCalculatorUtil.getDurationRound(value / (60.0 * hoursToDay) + ""));
        }
//        df.setMaximumFractionDigits(5);
//        return df.format(value);
        return value;

    }

    /**
     * 获取月初和月尾
     * 日期字符串
     *
     * @return
     */
    private static Map<String, String> getFirstAndLastDayMonth() {
        Map<String, String> map = new HashMap<>();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Calendar cale;
        String firstday, lastday;
        // 获取前月的第一天
        cale = Calendar.getInstance();
        cale.add(Calendar.MONTH, 0);
        cale.set(Calendar.DAY_OF_MONTH, 1);
        firstday = format.format(cale.getTime());
        map.put("firstDay", firstday);
        // 获取前月的最后一天
        cale = Calendar.getInstance();
        cale.add(Calendar.MONTH, 1);
        cale.set(Calendar.DAY_OF_MONTH, 0);
        lastday = format.format(cale.getTime());
        map.put("lastDay", lastday);
        return map;
    }

    /**
     * 判断某个值是否在某个区间内(区间闭合,包含所在的起始值和结束值)
     *
     * @param current 特定数值
     * @param min     区间起始位
     * @param max     区间结束位
     * @return true 在;false 不在
     */
    public static boolean rangeInDefined(double current, int min, int max) {
        return Math.max(min, current) == Math.min(current, max);
    }


    /**
     * 连续加班工作6天
     *
     * @param resourceid
     * @return
     */
    public static boolean isContinuousWork(String resourceid) {
        int continuityDays = 6;
        Set<String> workDayList = new HashSet<>();
        RecordSet rs = new RecordSet();
        // 查询近6天的数据
        String sql = "select distinct fromDate,toDate from kq_flow_overtime where date_sub(CURDATE(),INTERVAL " + continuityDays + " DAY) <= DATE(fromDate) and resourceid = ?";
        rs.executeQuery(sql, resourceid);
        while (rs.next()) {
            String fromDate = rs.getString("fromDate");
            String toDate = rs.getString("toDate");
            if (fromDate.equals(toDate)) {
                workDayList.add(fromDate);
            } else {
                while (!fromDate.equals(toDate) && workDayList.size() < continuityDays) {
                    workDayList.add(fromDate);
                    fromDate = DateUtil.addDate(fromDate, 1);
                }
                workDayList.add(toDate);
            }
        }
        return workDayList.size() == continuityDays;
    }

    public static String getMessageType() {
        return messageType;
    }

    public static void setMessageType(String messageType) {
        MessagePushWeChatWorkAction.messageType = messageType;
    }

    public static String getTitle() {
        return title;
    }

    public static void setTitle(String title) {
        MessagePushWeChatWorkAction.title = title;
    }

    public static String getLinkUrl() {
        return linkUrl;
    }

    public static void setLinkUrl(String linkUrl) {
        MessagePushWeChatWorkAction.linkUrl = linkUrl;
    }

    public static String getLinkMobileUrl() {
        return linkMobileUrl;
    }

    public static void setLinkMobileUrl(String linkMobileUrl) {
        MessagePushWeChatWorkAction.linkMobileUrl = linkMobileUrl;
    }
}
